#!/usr/bin/env python
# Written by: DGC

# python imports

# local imports
import qtGUI

def main():
    application = qtGUI.MainApp()
    application.MainLoop()

#==============================================================================
if (__name__ == "__main__"):
    main()
